@@ -23,15 +23,15 @@ static struct {
 	{ "ftp://user:password@host", "ftp://user@host",
 	  { "ftp", "user", "password", "host", 21, "", NULL, NULL } },
 	{ "http://us%65r@host", "http://user@host/",
-	  { "http", "user", NULL, "host", 80, "/", NULL, NULL } },
+	  { "http", "user", "", "host", 80, "/", NULL, NULL } },
 	{ "http://us%40r@host", "http://us%40r@host/",
-	  { "http", "us\x40r", NULL, "host", 80, "/", NULL, NULL } },
+	  { "http", "us\x40r", "", "host", 80, "/", NULL, NULL } },
 	{ "http://us%3ar@host", "http://us%3Ar@host/",
-	  { "http", "us\x3ar", NULL, "host", 80, "/", NULL, NULL } },
+	  { "http", "us\x3ar", "", "host", 80, "/", NULL, NULL } },
 	{ "http://us%2fr@host", "http://us%2Fr@host/",
-	  { "http", "us\x2fr", NULL, "host", 80, "/", NULL, NULL } },
+	  { "http", "us\x2fr", "", "host", 80, "/", NULL, NULL } },
 	{ "http://us%3fr@host", "http://us%3Fr@host/",
-	  { "http", "us\x3fr", NULL, "host", 80, "/", NULL, NULL } },
+	  { "http", "us\x3fr", "", "host", 80, "/", NULL, NULL } },
 	{ "http://host?query", "http://host/?query",
 	  { "http", NULL, NULL, "host", 80, "/", "query", NULL } },
 	{ "http://host/path?query=http%3A%2F%2Fhost%2Fpath%3Fchildparam%3Dchildvalue&param=value",
@@ -120,6 +120,14 @@ static struct {
 
 	{ "HTTP:////////////////", "http:////////////////",
 	  { "http", NULL, NULL, "", 80, "//////////////", NULL, NULL } },
+
+	{ "http://@host", "http://@host/",
+	  { "http", "", "", "host", 80, "/", NULL, NULL } },
+	{ "http://:@host", "http://@host/",
+	  { "http", "", "", "host", 80, "/", NULL, NULL } },
+
+	{ "http://host/keep%00nuls", "http://host/keep%00nuls",
+	  { "http", NULL, NULL, "host", 80, "/keep%00nuls", NULL, NULL } },
 };
 static int num_abs_tests = G_N_ELEMENTS(abs_tests);
 
